-------------------------------------------------------------------
-- Copyright (c) Doolali 2014. All rights reserved
-------------------------------------------------------------------
-- Special Thanks 
-- A thanks to some of the other addons that provided some functionality 
-- to what I've made!
---- Nexus Panel 	-- formatTime function
---- Rare Tracker 	-- GUI design
---- Goldbum		-- UI management
-------------------------------------------------------------------
-- VERSION HISTORY
  -- v1.05 - release of BG Shouter
  -- v1.06 - added queue timer
  --		 improved kill blow shouter
  -- v1.07 - fixed bug that stopped the shouter after a game  
  --		 fixed killing spree bug
  -- v1.08 - added death loop timer
  -- v1.09 - API upgrade to 9.
  -- v1.10 - added latency, ping and FPS values
  --         added shout buttons
  -- v1.11 - private release
  --		 added timer support for French and German
  --         added a streak counter that persists across games
  --		 UI is now position now saved
  -- v1.12 - curse release of 1.11
  -- v1.15 - the killing blow window is now ignorable by the mouse
  -- v1.16 - Fixed a bug in bloodsworn that caused an error to show
  --         when you join half way through and finish.
  -- v1.17 - Addon settings are now saved! :):)
  --	     Cleaned up some of the functions
  --	     Added heal tracking and basic system reporting (not live)
-------------------------------------------------------------------
require "Window"
require "MatchingGame"
  
 require "Apollo"
require "ChatSystemLib"
require "GameLib"
require "Unit"
require "Spell"
require "GroupLib"

  
-------------------------------------------------------------------
local BGShouter = {}

local normalTextColor = ApolloColor.new("UI_WindowTextDefault")
local selectedTextColor = ApolloColor.new("UI_BtnTextHoloPressedFlyby")
local activeTextColor = ApolloColor.new("xkcdAppleGreen")
local inactiveTextColor = ApolloColor.new("xkcdCherryRed")
local customUnitColor = ApolloColor.new("xkcdBloodOrange")

local version = "Version: 1.17 beta"

local author = "Author: Doolali@Ravenous Mavadosh@Ravenous"

local doolali = false
local debugM = false

local time = os.time()

local Started = false
local HalfWay = false

local StartTime = os.time()

local tikiDomScore = 1
local tikiExScore = 1

local FinishMessage
local FirstRound
local Attacking

local Timer
local ReportTimer
local TikiTimer
local RestoreDataTimer

local GameOver = false
local KnownMask = true
local DontShout = false
local hold = false
local shoutMuted = false

local dMaskStolen = false
local eMaskStolen = false


local deLevel
local dtSavedData

local BGTracker
local Tracking
local TrackingValue = 1

local lastHeal = 0
local totalHeals = 0
local healing100 = false
local healing250 = false
local healing500 = false
local healing1000 = false
local healing1500 = false
local healing2000 = false
local healing3000 = false


local currency =
{
	{
		eType = Money.CodeEnumCurrencyType.Prestige, 		
		strTitle = Apollo.GetString("CRB_Prestige"), 	
		strDescription = Apollo.GetString("CRB_Prestige_Desc")
	}
}

-------------------------------------------------------------------
-- Init
-------------------------------------------------------------------
function BGShouter:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 
	o.Tracking = {}
	o.killingBlows = 0
	o.deaths = 0
	o.killSpree = 0
	o.pKillSpree = 0
	o.hidden = false
    return o
end

function BGShouter:Init()	
	local bHasConfigureFunction = false
	local strConfigureButtonText = "BGShouter"
	
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, {})	
end

function BGShouter:OnLoad()	
	-- UI LOAD
	self.xmlDoc = XmlDoc.CreateFromFile("BGShouter.xml")	
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)	
	
	-- EVENT REGISTRATION
    Apollo.RegisterEventHandler("ChatMessage","OnChatMessage", self)
	Apollo.RegisterEventHandler("PvpKillNotification", "OnPvpKill", self)
	Apollo.RegisterEventHandler("MatchEntered", "OnMatchEntered", self)
	Apollo.RegisterEventHandler("MatchExited", "OnMatchExit", self)
	Apollo.RegisterEventHandler("MatchingGameReady", "OnGameReady", self)
	Apollo.RegisterEventHandler("CombatLogResurrect", "OnRessurect", self)
	Apollo.RegisterEventHandler("CombatLogHeal", "OnHeal", self)
	
	-- TIMER HANDLERS
	Apollo.RegisterTimerHandler("ClearKillAlert", "HideKillAlert", self)
	
	-- SLASH COMMANDS
  	Apollo.RegisterSlashCommand("bgs", "OnBGShouterOn", self)
  	Apollo.RegisterSlashCommand("BGShouter", "OnBGShouterOn", self)
	
	-- ACTIVE TIMERS
	-- create a timer to track the BG events (15 times a second)
    self.trackingTimer = ApolloTimer.Create(1/15, true, "OnTrackingTimer", self)
	-- do a prestigeCheck every 15 seconds
    self.prestigeCheckTimer = ApolloTimer.Create(15, true, "OnPrestigeCheckTimer", self)
	-- fps, ping and latency check	
    self.pingFPSCheckTimer = ApolloTimer.Create(1, true, "CheckFPSAndPing", self)
end

function BGShouter:OnDocLoaded()
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
    
		self.mainWindow = Apollo.LoadForm(self.xmlDoc, "BGShouterForm", nil, self)
		
		self.killingBlowOverlay = Apollo.LoadForm(self.xmlDoc, "KillingBlowOverlay", nil, self)
			
		if self.mainWindow == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
		if self.killingBlowOverlay == nil then
			Apollo.AddAddonErrorText(self, "Could not load the Killing Blow Overlay for some reason.")
			return
		end

		self.mainWindow:Show(false, true)  
		
		-- Start using the Carbine window management
		Apollo.RegisterEventHandler("WindowManagementReady", "OnWindowManagementReady", self)
		
		-- hide UI display elements
		self:DisplayElement(self.killingBlowOverlay, "KillingBlowDisplay", false)		
		self:DisplayElement(self.killingBlowOverlay, "KillingSpreeDisplay", false)
		self:DisplayElement(self.killingBlowOverlay, "UnstoppableDisplay", false)
		self:DisplayElement(self.killingBlowOverlay, "GodLikeDisplay", false)
		self:DisplayElement(self.killingBlowOverlay, "MonsterKillDisplay", false)
		
		self.BGTracker = self.mainWindow:FindChild("BGTracker")
				
		self.mainWindow:FindChild("MutedCheckButton"):SetCheck(not shoutMuted)
		
		if not self.hidden then
			self:OnShow()
		end
		
		self:prestigeCheck()	
		
		self:UpdateText(self.mainWindow, "KSDisplay", "" .. self.pKillSpree)
		
		--local captureTime = os.time()	
		--self:AddTracker("Respawn Timer", nil, captureTime)	
		--self:AddTracker("Respawn Timer", nil, captureTime)	
		--self:AddTracker("Respawn Timer", nil, captureTime)	
		
		self.killingBlowOverlay:Show(false)
	end
	sendSystemMessage("BGShouter loaded")
end

function BGShouter:OnSave(saveLevel)
	if saveLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return nil
	end
	
	local savedData = {}

	savedData.pKillSpree = self.pKillSpree
	savedData.hidden = self.hidden
	savedData.shoutMuted = shoutMuted 

	return savedData
end

function BGShouter:OnRestore(saveLevel, savedData)
	if savedData.pKillSpree ~= nil then
		self.pKillSpree = savedData.pKillSpree
	end
	
	if savedData.hidden ~= nil then
		self.hidden = savedData.hidden
	end

	if savedData.shoutMuted ~= nil then
		shoutMuted = savedData.shoutMuted
	end
end

-- Add the UI into the Carbine window management
-- Thanks Goldblum!
function BGShouter:OnWindowManagementReady()
	Event_FireGenericEvent("WindowManagementAdd", {wnd = self.mainWindow, strName = "BGShouter_0"})
end

-------------------------------------------------------------------
-- GUI Actions
-------------------------------------------------------------------

function BGShouter:OnClose()
  self.mainWindow:Close()
  self.hidden = true
end

function BGShouter:OnShow()
	self.mainWindow:Invoke()
end

-- function for listening to slash commands
function BGShouter:OnBGShouterOn(slash, message)	
	if message == "show" then
		self.hidden = false
		self:OnShow()
		return
	end
	if message == "hide" then
		self:OnClose()
		return
	end
	if message == "close" then
		self:OnClose()
	return
	end
	if message == "version" then
		sendSystemMessage(author)
		sendSystemMessage(version)
		return
	end	
	if message == "author" then
		sendSystemMessage(author)
		sendSystemMessage(version)
		return
	end
	if message == "update" then
		self:prestigeCheck()
		return
	end	
	if message == "start" then
		self.killingBlowDisplay:Show(true)
		return
	end	
	if message == "end" then
		self:HideKillingBlow()
		return
	end	
	
	sendSystemMessage("BGShouter usage:")
	sendSystemMessage("/bgs version - shows the addon version")
	sendSystemMessage("/bgs author 	- shows the addon author")
	sendSystemMessage("/bgs hide 	- hides the addon")
	sendSystemMessage("/bgs show 	- shows the addon")
end

-------------------------------------------------------------------
-- Game Logic
-------------------------------------------------------------------

function BGShouter:OnRessurect()
	sendDebugMessage("I HAVE RESURRECTED!")
	--self:AddTracker("Respawn Timer", nil, os.time())
end

function BGShouter:OnGameReady()
	local captureTime = os.time()					
	sendDebugMessage(captureTime);
	self:AddTracker("Queue popup", nil, captureTime)	
end

function BGShouter:OnMatchEntered()
	self:OnShow()
	self:ClearTracker()
	self.killingBlows = 0
	self.deaths = 0
	self.killSpree = 0
	totalHeals = 0
	self:SetKillBlows()
	GameOver = false
	KnownMask = true
end

function BGShouter:OnMatchExit()

	self:ClearTracker()
	self.killingBlows = 0
	self.deaths = 0
	self.killSpree = 0
	self:SetKillBlows()
	totalHeals = 0
	GameOver = false
	
	if self.hidden then
		self:OnClose()
	end
end

function BGShouter:OnPvpKill(victim, cause, killer, killerClass, victimTeam)
	local me = GameLib.GetPlayerUnit()
	local myName = me:GetName()
	
	local enemyName = ""
	
	-- make sure the player playing is the killer
	if killer == myName then		
		enemyName = victim
		Sound.Play(197)
		self.killingBlows = self.killingBlows + 1
		Apollo.CreateTimer("ClearKillAlert", 3, false)
				
		-- increase spree count
		self.killSpree = self.killSpree + 1
		
		-- increase kill spree that lasts through BGs
		self.pKillSpree = self.pKillSpree + 1
		
		-- set who you killed
		self:UpdateText(self.killingBlowOverlay, "PlayerKilledDisplay", victim)

		-- did we get a spree?
		if self.killSpree == 3 then
			-- killing spree
			self:DisplayElement(self.killingBlowOverlay, "KillingSpreeDisplay", true)	
			sendEmoteMessage("is on a KILLING SPREE!")
		elseif self.killSpree == 5 then
			-- unstoppable
			self:DisplayElement(self.killingBlowOverlay, "UnstoppableDisplay", true)		
			sendEmoteMessage("is UNSTOPPABLE!!")
		elseif self.killSpree == 7 then
			-- god like
			self:DisplayElement(self.killingBlowOverlay, "GodLikeDisplay", true)
			sendEmoteMessage("is GOD LIKE!!!")
		elseif self.killSpree == 20 then
			-- M M M MONSTER KILL!
			self:DisplayElement(self.killingBlowOverlay, "MonsterKillDisplay", true)
			sendEmoteMessage("JUST HAD A M M M MONSTER KILL!!!!")
		else			
		end
		
		self:DisplayElement(self.killingBlowOverlay, "KillingBlowDisplay", true)
		self.killingBlowOverlay:Show(true)
		
		self:CheckDominating()
	end
	
	if victim == myName then	
		enemyName = killer
		self.deaths = self.deaths + 1
		
		self.killSpree = 0
		self.pKillSpree = 0
				
		self:CheckDominating()
	end
	
	self:SetKillBlows()
end

function BGShouter:CheckDominating()
	
end

function BGShouter:HideKillAlert()
	self.killingBlowOverlay:Show(false)
	
	self:DisplayElement(self.killingBlowOverlay, "KillingBlowDisplay", false)
		
	self:DisplayElement(self.killingBlowOverlay, "KillingSpreeDisplay", false)
	self:DisplayElement(self.killingBlowOverlay, "UnstoppableDisplay", false)
	self:DisplayElement(self.killingBlowOverlay, "GodLikeDisplay", false)
	self:DisplayElement(self.killingBlowOverlay, "MonsterKillDisplay", false)
end

function BGShouter:SetKillBlows()
	-- update the kill death display
	self:UpdateText(self.mainWindow, "KBValue", "" .. string.format("%d/%d", self.killingBlows, self.deaths))
	-- update the kill streak display
	self:UpdateText(self.mainWindow, "KSDisplay", "" .. self.pKillSpree)
end


function IncreaseTikiAverage()
	--nBGShouter
end

function BGShouter:OnChatMessage(channelCurrent, tMessage)
	local message = tMessage.arMessageSegments[1].strText
	
	if channelCurrent:GetName() == "Instance" then	
		self:checkVersion(message)
	end

	if 
		channelCurrent:GetName() == "System" or -- english/german
		channelCurrent:GetName() == "Système" -- french
	then	
		self:gameCheck(message)	
	end

	if 
		channelCurrent:GetName() == "System" or -- english/german
		channelCurrent:GetName() == "Système" -- french
	then	
		if
			message == "You have been kicked by the server." or -- english
			string.find(message, "Vous avez été expulsé par le serveur") ~= nil or -- french
			message == "Deine Verbindung mit dem Server wurde getrennt." -- german
		then
			if self.hidden then
				self:OnClose()
			end
			self:ClearTracker()
			self:prestigeCheck()
		end	
	end

	if 
		channelCurrent:GetName() == "Datachron" or -- english
		channelCurrent:GetName() == "Infochron" -- german/french
	then		
		sendDebugMessage("datachron")
		self:scoreCount(message)				
		self:moodieMaskCheck(message)		
		self:bloodsworn(message)
	end
end

function BGShouter:checkVersion(message)
	--if string.find(message, version) then
		--DontShout = true
		--sendSystemMessage("Another player has BGShouter they will shout in this match. ")
	--end
end

function BGShouter:bloodsworn(message)

	if message == "You are defending." then
		if Started == false then
			StartTime = os.time()
			Started = true
			self:shoutVersion()
		end
	end
	
	if message == "You are attacking." then
		if Started == false then
			StartTime = os.time()
			Started = true
			Attacking = true
			self:shoutVersion()
		end
	end

	if message == "Round Complete. Teams will now switch sides." then
		if HalfWay == false then
			HalfWay = true
		
			FirstRound = formatTime(os.time() - StartTime)
			
			if DontShout == false then
				sendInstanceMessage("First round took: " .. FirstRound)
			end
			
			StartTime = os.time()
		end
	end
	if string.find(message, "You have been DEFEATED.") then
		local SecondRound = formatTime(os.time() - StartTime)
	
		sendDebugMessage("Second round took: " .. formatTime(os.time() - StartTime))
		
		if FirstRound ~= nil then
			if Attacking then
				FinishMessage = "You attacked in " .. FirstRound .. " and were defeated!"
			else				
				FinishMessage = "You failed to beat a time of " .. FirstRound .. "!"
			end			
			-- only report if we have the message
			self:DelayedReport()
		end
	end
	
	if message == "You are VICTORIOUS!" then

		local SecondRound = formatTime(os.time() - StartTime)
			
		sendDebugMessage("Second round took: " .. formatTime(os.time() - StartTime))
		
		if FirstRound ~= nil then
			FinishMessage = "You attacked in " .. FirstRound .. " and were victorious!"
			
			-- only report if we have the message
			self:DelayedReport()
		end				
	end
		
end

function BGShouter:moodieMaskCheck(message)
	if 
	message == "The Dominion captured a Moodie Mask!" or message == "The Exiles captured a Moodie Mask!" or -- english
	string.find(message, "Le Dominion a capturé un masque moodie") ~= nil or string.find(message, "Les Exilés ont capturé un masque moodie") ~= nil or -- french
	message == "Das Dominion hat eine Moodiemaske erobert!" or message == "Die Verbannten haben eine Moodiemaske erobert!" -- german
	then	
		sendDebugMessage("Mask Captured")
		if GameOver == false then		
			sendDebugMessage("Game not over make mask")
			local captureTime = os.time()							
			local nextSpawn2 = math.floor((1738 + (StartTime - captureTime)))							
			printTime(nextSpawn2)	
			self:AddTracker("Mask", nil, captureTime)	
			if not KnownMask then
				self.UnknownMaskTimer = ApolloTimer.Create(49, false, "UnknownMaskSpawnSoon", self)
			else
				self.KnownMaskTimer = ApolloTimer.Create(49, false, "KnownMaskSpawnSoon", self)
			end
		end	
	else 
		if 
		message == "A Moodie Mask has been unearthed!" or -- english
		string.find(message, "Un masque moodie a été déterré") ~= nil or -- french
		message == "Eine Moodiemaske wurde ausgegraben!"    -- german
		then
			if Started == false then			
				Started = true
				StartTime = os.time()
				sendDebugMessage("PVP match started: " .. os.date("%X", StartTime))
				if doolali then
					sendInstanceMessageAlways("BGShouter! Now on Curse!")
				end
				self:shoutVersion()				
			end	
			
			self:ClearTracker()
		end
	end

end

function BGShouter:AddTracker(name, value, startTime)		
	local track = Apollo.LoadForm(self.xmlDoc, "Tracker", self.BGTracker, self)
	
	sendDebugMessage("tracker added")
	
	self:UpdateText(track, "Name", name)
	
	if value ~= nil then
		self:UpdateText(track, "Value", value)
	else 		
		local currentTime = os.time()		
		self:UpdateText(track, "Value", formatTime(60 - (currentTime - startTime)))	
	end
	
	local currentNType = "mask"
	
	if name == "Respawn Timer" then 
		currentNType = "respawn"
		Tracking = 0
		sendDebugMessage("RT")
	end
	
	self.Tracking[TrackingValue] = {
		track = track,
		startTime = startTime,
		name = TrackingValue,
		nType = currentNType
	}
  
	TrackingValue = TrackingValue + 1
	
	self.BGTracker:ArrangeChildrenVert()
end

function BGShouter:OnTrackerClick(windowHandler, windowControl, mouseButton)
  if windowHandler ~= windowControl then
    return
  end 
  
	if mouseButton == GameLib.CodeEnumInputMouse.Left then
		local item = windowControl:FindChild("Name"):GetText()
		local value = windowControl:FindChild("Value"):GetText()
		if item == "Mask" then
			sendInstanceMessageAlways("Mask spawning in " .. value)
		end
	end

  if mouseButton == GameLib.CodeEnumInputMouse.Right then
    self:RemoveTracker(windowControl)
  end
end

function BGShouter:RemoveTracker(windowControl)
  windowControl:Destroy()    
  self.BGTracker:ArrangeChildrenVert()
end

function BGShouter:OnTrackingTimer()
	for idx,item in pairs(self.Tracking) do
		if item ~= nil then
			if item.startTime ~= nil then
				-- get the current time for comparison
				local currentTime = os.time()		
				
				local timerTime = 60
				
				if item.nType == "respawn" then
					timerTime = 29
				end
				
				-- work out how much time we have left until the tiki mask spawns
				local maskTimeLeft = timerTime - (currentTime - item.startTime)
				
				-- only update the time on positive values
				if maskTimeLeft > -1 then
					try(function()
						-- update the timer until we reach 0
						self:UpdateText(item.track, "Value", formatTime(maskTimeLeft))	
					end, function(e)
						sendDebugMessage("NIL START")
						item.startTime = nil;
					end)
					
					if maskTimeLeft < 20 then
						if item.nType == "respawn" then
							hold = false							
						end
					end					
				else
					if item.nType == "respawn" then
						item.startTime = nil
						self:RemoveTracker(item.track)
						if hold == false then
							hold = true							
							self:AddTracker("Respawn Timer", nil, os.time())
						end
					else
						-- remove items that have timed out
						self:RemoveTracker(item.track)
					end
				end
			end
		end
	end
end

function BGShouter:ClearTracker()
	-- loop through all items
	for idx,item in pairs(self.Tracking) do
		if item ~= nil then
			self:RemoveTracker(item.track)
		end
	end
	self.Tracking = {}
end


function BGShouter:shoutVersion()
	--sendInstanceMessage(version)
end

function BGShouter:gameCheck(message)
	if
		message == "You have left the matchmaking queue."  or -- english
		string.find(message, "Vous n'êtes plus en attente d'autres joueurs") ~= nil  or -- french
		message == "Du hast die Warteschlange für die Gegnersuche verlassen"  -- german
	then	
		self:resetBG()
		
		if self.hidden then
			self:OnClose()
		end
	end
end

function BGShouter:resetBG()
	healing100 = false
	healing250 = false
	healing500 = false
	healing1000 = false
	healing1500 = false
	healing2000 = false
	Started = false
	tikiDomScore = 1
	tikiExScore = 1
	DontShout = false
	FinishMessage = nil
	self:ClearTracker()	
	self:OnMatchEntered()
end

function BGShouter:KnownMaskSpawnSoon()
	sendInstanceMessage("Mask spawning in ELEVEN seconds!")
end

function BGShouter:UnnownMaskSpawnSoon()
	sendInstanceMessage("Mask might be spawning in ELEVEN seconds!")
end

function BGShouter:OnPrestigeCheckTimer()
	self:prestigeCheck()
end

function BGShouter:prestigeCheck(printWarning)
	local amount = GameLib.GetPlayerCurrency(currency[1].eType):GetAmount()
		
	if(self.mainWindow == nil) then
		sendDebugMessage("nil window")
	else
		local prestigeValueText = self.mainWindow:FindChild("PrestigeValue")
		
		prestigeValueText:SetText(string.format("%d", amount))
			
		if amount > 4000 then
			prestigeValueText:SetTextColor(inactiveTextColor)
		else		
			prestigeValueText:SetTextColor(normalTextColor)
		end
	end
end

function BGShouter:scoreCount(message)	
	if
		message == "The Dominion captured a Moodie Mask!" or -- english
		string.find(message, "Le Dominion a capturé un masque moodie") ~= nil or -- french
		message == "Das Dominion hat eine Moodiemaske erobert!" -- german
	then
		tikiDomScore = tikiDomScore + 1
		eMaskStolen = false
	end
	
	if
		message == "The Exiles captured a Moodie Mask!" or -- english
		string.find(message, "Les Exilés ont capturé un masque moodie") ~= nil or -- french
		message == "Die Verbannten haben eine Moodiemaske erobert!" -- german
	then		
		tikiExScore = tikiExScore + 1
		dMaskStolen = false
	end
	
	if
		message == "The Dominion recovered a stolen Mask!" or -- english
		string.find(message, "Le Dominion a récupéré un masque volé") ~= nil or -- french
		message == "Das Dominion hat eine gestohlene Maske zurückgeholt!" -- german
	then
		tikiDomScore = tikiDomScore + 1	
		dMaskStolen = false
	end
	
	if
		message == "The Exiles recovered a stolen Mask!" or -- english
		string.find(message, "Les Exilés ont récupéré un masque volé") ~= nil or -- french
		message == "Die Verbannten haben eine gestohlene Maske zurückgeholt!" -- german
	then
		tikiExScore = tikiExScore + 1	
		eMaskStolen = false
	end
	
	if 
		message == "The Dominion stole a Moodie Mask!" or -- english
		string.find(message, "Le Dominion a volé un masque moodie") ~= nil or -- french
		message == "Das Dominion hat eine Moodiemaske gestohlen!" -- german
	then
		tikiExScore = tikiExScore - 1
		eMaskStolen = true
	end
	
	if 
		message == "The Exiles stole a Moodie Mask!" or -- english
		string.find(message, "Les Exilés ont volé un masque moodie") ~= nil or -- french
		message == "Die Verbannten haben eine Moodiemaske gestohlen!" -- german
	then	
		tikiDomScore = tikiDomScore - 1
		dMaskStolen = true
	end	
	
	if dMaskStolen == false and eMaskStolen == false then
		--sendSystemMessage("KNOWN MASK")
		KnownMask = true
	else
		--sendSystemMessage("UNKNOWN MASK")
		KnownMask = false
	end
	
	if tikiDomScore == 5 then
		local EndTime = os.time()	
		sendDebugMessage("FinishMessage")
		FinishMessage = ("The Dominion won in " .. formatTime(EndTime - StartTime))
		--IncreaseTikiAverage(EndTime - StartTime)		
		sendDebugMessage(FinishMessage)
		self:DelayedReport()
	end
	
	if tikiExScore == 5 then
		local EndTime = os.time()		
		sendDebugMessage("FinishMessage")
		FinishMessage = ("The Exiles won in " .. formatTime(EndTime - StartTime))
		sendDebugMessage(FinishMessage)
		self:DelayedReport()
	end

end

function BGShouter:DelayedReport()
	if GameOver == false then		
		--sendSystemMessage(totalHeals)
		sendInstanceMessage(FinishMessage)	
		sendSystemMessage(FinishMessage)	
		self:prestigeCheck()
		ReportTimer = nil
		self:ClearTracker()
		GameOver = true
	end
end

function BGShouter:CheckFPSAndPing()
	self:UpdateText(self.mainWindow, "FPSDisplay", "FPS: " .. math.floor(GameLib.GetFrameRate()))
	self:UpdateText(self.mainWindow, "PingDisplay", "Ping: " .. GameLib.GetPingTime())
	self:UpdateText(self.mainWindow, "LatDisplay", "Latency: " .. GameLib.GetLatency())
end

-------------------------------------------------------------------
--  HEAL COUNTER
-------------------------------------------------------------------

function BGShouter:OnHeal(heal)
	-- make sure the unitCaster is not null (not sure what causes this, but something does!)
	if(heal.unitCaster ~= nil) then
	
		-- only count the heals from yourself!
		if heal.unitCaster:GetName() ==  GameLib.GetPlayerUnit():GetName() then			
		
			-- get the amount of actual healing for this event
			local healAmount = heal.nHealAmount
			
			-- if the overhealing matches the healing, we ignore it! I know shocking!
			if healAmount == heal.nOverheal then
				healAmount = 0
			end
			
			-- make sure we aren't counting duplicates (strange bug not sure why this happens)
			if healAmount ~= lastHeal and healAmount > 0 then
			
				-- set the last heal as the current to help prevent heal duplication
				lastHeal = healAmount		
				
				-- count up the healing done (ignoring overheals)
				totalHeals = totalHeals + healAmount
				
				-- print a message given the category of heal 
				-- dont repeat messages more than once
				if totalHeals > 100000 and not healing100 then
					healing100 = true
					--sendSystemMessage("100k healing reached!")
				end
				if totalHeals > 250000 and not healing250 then
					healing250 = true
					--sendSystemMessage("250k healing reached!")
				end
				if totalHeals > 500000 and not healing500 then
					healing500 = true
					--sendSystemMessage("500k healing reached!")
				end
				if totalHeals > 1000000 and not healing1000 then
					healing1000 = true
					--sendSystemMessage("1 million healing reached!")
				end
				if totalHeals > 1500000 and not healing1500 then
					healing1500 = true
					--sendSystemMessage("1.5 million healing reached!")
				end
				if totalHeals > 2000000 and not healing2000 then
					healing2000 = true
					--sendSystemMessage("2 million healing reached!")
				end
				if totalHeals > 3000000 and not healing3000 then
					healing3000 = true
					--sendSystemMessage("3 million healing reached!")
				end
			end			
		end
	end
end

-------------------------------------------------------------------
--  SHOUTER BUTTON EVENTS
-------------------------------------------------------------------
function BGShouter:OurCaveTouched()
	sendInstanceMessageAlways("FOCUS: Our CAVE")
end

function BGShouter:EnemyCaveTouched()
	sendInstanceMessageAlways("FOCUS: Enemy CAVE")
end

function BGShouter:RightTouched()
	sendInstanceMessageAlways("FOCUS: RIGHT")
end

function BGShouter:LeftTouched()
	sendInstanceMessageAlways("FOCUS: LEFT")
end

function BGShouter:MidTopTouched()
	sendInstanceMessageAlways("FOCUS: Middle TOP")
end

function BGShouter:MidBotTouched()
	sendInstanceMessageAlways("FOCUS: Middle BOTTOM")
end

function BGShouter:NeutralResetTouched()
end

function BGShouter:OurBaseTouched()
	sendInstanceMessageAlways("FOCUS: Our BASE")
end

function BGShouter:TheirBaseTouched()
	sendInstanceMessageAlways("FOCUS: Their BASE")
end

function BGShouter:onMute()
	if shoutMuted == false then
		shoutMuted = true
		sendSystemMessage("BGShouter has been muted.")
	else
		shoutMuted = false
		sendSystemMessage("BGShouter has been unmuted.")
	end
end

-------------------------------------------------------------------
-- UI Helpers
-------------------------------------------------------------------

function BGShouter:UpdateText(item, element, text)	
	if item ~= nil then
		if element ~= nil then
			item:FindChild(element):SetText("" .. text)
		end
	end
end

function BGShouter:DisplayElement(item, element, visible)
	item:FindChild(element):Show(visible)
end

-------------------------------------------------------------------
-- LUA Helpers
-------------------------------------------------------------------
function try(f, catch_f)
	local status, exception = pcall(f)
	if not status then
		catch_f(exception)
	end
end

-------------------------------------------------------------------
--  CHAT HELPERS
-------------------------------------------------------------------
function sendInstanceMessage(message)
	if shoutMuted == false then
		for _,channel in pairs(ChatSystemLib.GetChannels()) do
			if channel:GetType() == ChatSystemLib.ChatChannel_Instance then
				channel:Send("[BGShouter] " .. message)
			end
		end
	end
end

function sendInstanceMessageAlways(message)
	for _,channel in pairs(ChatSystemLib.GetChannels()) do
		if channel:GetType() == ChatSystemLib.ChatChannel_Instance then
			channel:Send("[BGShouter] " .. message)
		end
	end
end

function sendSayMessage(message)
	if shoutMuted == false then
		for _,channel in pairs(ChatSystemLib.GetChannels()) do
			if channel:GetType() == ChatSystemLib.ChatChannel_Say then
				channel:Send("[BGShouter] " .. message)
			end
		end
	end
end

function sendEmoteMessage(message)
	ChatSystemLib.Command("/e " .. message)
end

function sendSystemMessage(message)
	ChatSystemLib.PostOnChannel(2,"[BGShouter] " .. message)
end

function sendDebugMessage(message)	
	if debugM then 
		ChatSystemLib.PostOnChannel(3,"[BGShouter]: " .. message)
	end
end

function printTime(nTime)	
	if not KnownMask then
		sendInstanceMessage("Potential Mask Spawn: " .. formatTime(nTime))
	else
		sendInstanceMessage("Mask Spawn: " .. formatTime(nTime))
	end
end

-- from Nexus Panel -- Thanks!
function formatTime(nTimeLeft)
	if type(nTimeLeft) ~= "number" then return nTimeLeft end
	local nDays, nHours, nMinutes, nSeconds = math.floor(nTimeLeft / 86400), math.floor((nTimeLeft % 86400) / 3600), math.floor((nTimeLeft % 3600) / 60), nTimeLeft % 60;

	if nDays ~= 0 then
		return ("%dd %dh %dm %ds"):format(nDays, nHours, nMinutes, nSeconds)
	elseif nHours ~= 0 then
		return ("%dh %dm %ds"):format(nHours, nMinutes, nSeconds)
	elseif nMinutes ~= 0 then
		return ("%dm %ds"):format(nMinutes, nSeconds)
	else
		return ("%ds"):format(nSeconds)
	end
end

local BGShouterInst = BGShouter:new()
BGShouterInst:Init()
